(ns com.aurellem.save-corruption
  (:use (com.aurellem gb-driver vbm title)))

(use 'clojure.repl)

(defn-memo start-walking [] 
  (->> (finish-title)
       (advance [:b] [:b :r])))

(def walk (partial advance []))

(defn-memo walk-to-stairs []
  (->> (start-walking)
       (walk [:u])
       (walk [:u])
       (walk [:u])
       (walk [:u])
       (walk [:u])
       (walk [:r])
       (walk [:r])
       (walk [:r])))

(defn-memo walk-to-door []
  (->> (walk-to-stairs)
       (walk [:d])
       (walk [:d])
       (walk [:d])
       (walk [:d])
       (walk [:d])
       (walk [:d])
       (walk [:l])
       (walk [:l])
       (walk [:l])
       (walk [:l])))
  

(defn-memo activate-menu []
  (->> (walk-to-door)
       (advance [:b] [:a :b :start])))

(defn-memo save-game []
  (->> (activate-menu)
       (advance [] [:d])
       (play-moves [[] [] [] [:d] [] [] [] [:d] [] [] [:a]])
       scroll-text))

(defn-memo corrupt-save []
  (->> (save-game)
       (play-moves
        ;; this section is copied from speedrun-2942
        ;; and corrupts the save so that the end-of-list marker
        ;; for the pokemon roster is destroyed, but the save is still
        ;; playable.
         [[] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
         [] [] [] [] [] [] [] [] [] [] [:select] [:restart]])))

(defn-memo skip-title-again []
  (->> (corrupt-save)
       (play-moves
        (first (title)))))

(defn-memo start-game []
  (->> (skip-title-again)
       (advance [] [:start])
       (advance [] [:a])))       

(defn warp-to-elite-four
  "This is copied from speedrun-2942 to ensure that everything is good
   up to this point."
  []
  (->> (corrupt-save)
       (play-moves
        [ [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [:start] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [:start] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a]
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [:start] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:d]
        [] [] [] [:a] [] [] [:d] [] [] [:a] [] [] [] [] [] [] [] [] []
        [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] []
        [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] [] [] [] [:d] [] []
        [] [] [:d] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [] [:a] [] [] [:d]
        [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] [] [] [:u] [] []
        [] [] [:u] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:b] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [:a] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [] [:d] []
        [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] []
        [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] []
        [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] []
        [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] []
        [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] []
        [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d]
        [] [] [] [] [] [] [] [] [] [] [] [] [] [:b] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:select] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [:b] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [] [] [] [] []
        [] [:b] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [:select] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:b] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:d] [] []
        [] [] [] [] [] [] [] [] [] [:b] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [:d] [] [] [] [] [:b] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:select] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:b] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [:d] [] [] [] [] [] [] []
        [:d] [] [] [] [] [] [] [] [:d] [] [] [] [] [] [] [:d] [] [] []
        [] [] [] [:d] [] [] [] [] [] [] [] [] [] [] [] [:select] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [:a] [] [] [:d] [] [] [:a] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u]
        [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] []
        [:u] [] [:u] [] [:u] [] [:u] [] [:u] [] [:u] [:a] [] [] [] []
        [] [:a] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:b] [] [] [] [] []
        [] [] [] [:b] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d] [:d]
        [:d] [:d] [:d] [:d] [:d] [:d] [:d] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:a] [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:a] [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] []
        [:a] [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a] []
        [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a] []
        [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [:a] [] [] [] []
        [] [] [] [] [] [] [] [] [:a] [] [] [] [] [] [] [] [] [] [] []
        [] [] [] [:b]])))

        
       


